#pragma rtGlobals=1		// Use modern global access method.


//functions are faster than macros
//functions can be inserted into the macros menu
//Proc procedures can use global wave names directly
//proc are macros that do not show up in the menu by default



//This function reads a generic SCL binary data file written by Jesus Rosario's
//Lab View data acquisition software. This data format is specified in
//NSWCCD-65-TR-2006/15 JHSV Analysis Engine
function loadGenericSCLFileSP4BHBF(fname)
	String fname //data file name input

	Variable fileref,numchan,headersize,scanrate,cal,i,deltat //local variables

	//Open file for read; will prompt if file does not exist.
	Open/R/T=".scl"/Z=2 fileref as fname

	Open/R/Z=0 fileref as S_fileName
	print "file name: " + S_fileName

	//FBinRead /B=2 high byte first /B=3 low byte first
	FBinRead/B=2/F=3 fileref,numchan
	print "number of channels: " + num2str(numchan)
	headersize = (numchan + 1 )*8
	print "header size: " + num2str(headersize)
	
	FBinRead/B=2/F=4 fileref,scanrate
	print "scan rate: " + num2str(scanrate)
	deltat = 1/scanrate
	
	for(i=0; i < numchan; i+=1)
		FBinRead/B=2/F=5 fileref,cal
		print "channel " + num2str(i) + " cal: " + num2str(cal)
	endfor

	Close fileref

	//load binary waves GBLoadWave /B=0 high byte first /B=1 low byte first
	execute "GBLoadWave/B=0/O/V/N=wave/T={2,2}/S=" + num2str(headersize) + "/W=" + num2str(numchan) + " \"" + S_fileName + "\""
	
	string swavename
	for(i=0; i < numchan; i+=1)
		swavename = "wave" + num2str(i)
		setscale/P x 0,deltat,"",$swavename
	endfor

end // end loadGenericSCL








//This function reads a generic SCL binary data file written by Jesus Rosario's
//Lab View data acquisition software. This data format is specified in
//NSWCCD-65-TR-2006/15 JHSV Analysis Engine, but low byte first version
function loadGenericSGLFileSP4BLBF(fname)
	String fname

	Variable fileref,numchan,headersize,scanrate,cal,i,deltat

	//Open file for read; will prompt if file does not exist.
	Open/R/T=".scl"/Z=2 fileref as fname

	Open/R/Z=0 fileref as S_fileName
	print "file name: " + S_fileName

	//FBinRead /B=2 high byte first /B=3 low byte first
	FBinRead/B=3/F=3 fileref,numchan
	print "number of channels: " + num2str(numchan)
	headersize = (numchan + 1 )*8
	print "header size: " + num2str(headersize)
	
	FBinRead/B=3/F=4 fileref,scanrate
	print "scan rate: " + num2str(scanrate)
	deltat = 1/scanrate
	
	for(i=0; i < numchan; i+=1)
		FBinRead/B=3/F=5 fileref,cal
		print "channel " + num2str(i) + " cal: " + num2str(cal)
	endfor

	Close fileref

	//load binary waves GBLoadWave /B=0 high byte first /B=1 low byte first
	execute "GBLoadWave/B=1/O/V/N=wave/T={2,2}/S=" + num2str(headersize) + "/W=" + num2str(numchan) + " \"" + S_fileName + "\""
	
	string swavename
	for(i=0; i < numchan; i+=1)
		swavename = "wave" + num2str(i)
		setscale/P x 0,deltat,"",$swavename
	endfor
	
end //loadGenericSGL












//This function reads a ScrewedUp generic SCL binary data file written by Jesus Rosario's
//Lab View data acquisition software. This data format is NOT specified in
//NSWCCD-65-TR-2006/15 JHSV Analysis Engine
//This crap data file has an extra time tag channel that is not specified in the binary header.
function loadScrewedUpSCLFileSP4BHBF(fname)
	String fname //data file name input

	Variable fileref,numchan,headersize,scanrate,cal,i,deltat //local variables

	//Open file for read; will prompt if file does not exist.
	Open/R/T=".scl"/Z=2 fileref as fname

	Open/R/Z=0 fileref as S_fileName
	print "file name: " + S_fileName

	//FBinRead /B=2 high byte first /B=3 low byte first
	FBinRead/B=2/F=3 fileref,numchan
	print "number of channels: " + num2str(numchan)
	headersize = (numchan + 1 )*8
	print "header size: " + num2str(headersize)
	
	FBinRead/B=2/F=4 fileref,scanrate
	print "scan rate: " + num2str(scanrate)
	deltat = 1/scanrate
	
	for(i=0; i < numchan; i+=1)
		FBinRead/B=2/F=5 fileref,cal
		print "channel " + num2str(i) + " cal: " + num2str(cal)
	endfor

	Close fileref

	//load binary waves GBLoadWave /B=0 high byte first /B=1 low byte first
	execute "GBLoadWave/B=0/O/V/N=wave/T={2,2}/S=" + num2str(headersize) + "/W=" + num2str(numchan+1) + " \"" + S_fileName + "\""
	
	string swavename
	for(i=0; i < numchan; i+=1)
		swavename = "wave" + num2str(i)
		setscale/P x 0,deltat,"",$swavename
	endfor

end // end loadScrewedUpSCL
















//scale, kill, rename
//this is a support function for writing
//data import functions
function skr(waven,scalefactor,deltat,newname)
	string waven
	variable scalefactor
	variable deltat
	string newname

	wave wvr = $waven
	wvr=wvr*scalefactor
	SetScale/P x 0,deltat,"", wvr
	killwaves/Z $newname
	Rename wvr, $newname

end








//this function was written for ddx to help
//subtract the mean from a run
//functions are faster than macros
//functions can be inserted into the macros menu though
Function SubtractMean(wname)
	String wname //stores the selected wave name
	if(stringmatch( wname, ""))//check to see if we are calling this from the macro menu
		Prompt wname,"Wave",popup,WaveList("*", ";", "") //promps from waves in the working data folder
		variable V_Flag
		DoPrompt "Wave",wname //creates the prompt window
		if (V_Flag)
			print "cancelled"
			return 0	// check if user canceled
		endif
	endif

	variable scans = numpnts($wname) //get the number of scans from the selected wave

	variable wmean = mean($wname,0,scans-1)


	wave waveref = $wname

	waveref=waveref-wmean

End //end Subtract Mean











//generic condition load
//has selections for every different ship, model, and data set
//filenames are explicitly created so files load in order
Function nswccd65LoadDataSetsFromFolder()

	variable /g singlechannel
	singlechannel = -1
	variable tempsinglechannel = singlechannel
	Prompt tempsinglechannel, "-1 for all channels, otherwise pick single channel"

	string /g splicemethod
	string tempsplicemethod = splicemethod
	Prompt tempsplicemethod,"splice method", popup, "none;subtractmean;forcecontinuous"

	string /g dataset
	string tempdataset = dataset
	Prompt tempdataset, "data set ", popup, "ddxsdmedium;ddxsdfast;ddxfld2slow;ddxfld2medium;ddxfld2fast;inls69;inls68;inls42;ddxbdslow;ddxfldslow;ddxmedium;ddxfast;ddxsternramp;hsv2slow;hsv2fast"
	
	variable V_Flag
	DoPrompt "Data Set Info", tempsinglechannel, tempsplicemethod, tempdataset
	if(V_Flag)
		print "cancelled"
		return 0
	endif

	singlechannel = tempsinglechannel
	splicemethod = tempsplicemethod
	dataset = tempdataset


	String /g prefix
	string /g extension
	variable /g filelowerbound
	variable /g fileupperbound






			if(stringmatch(dataset , "ddxbdslow"))
				prefix = "Slow_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 30
			elseif(stringmatch(dataset , "ddxfldslow"))
				prefix = "Slow_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 30
			elseif(stringmatch(dataset , "ddxmedium"))
				prefix = "Med_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 30
			elseif(stringmatch(dataset , "ddxfast"))
				prefix = "Fast_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 30
			elseif(stringmatch(dataset , "ddxfld2slow"))
				prefix = "Slow_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 100
			elseif(stringmatch(dataset , "ddxfld2medium"))
				prefix = "Med_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 100
			elseif(stringmatch(dataset , "ddxfld2fast"))
				prefix = "Fast_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 100
			elseif(stringmatch(dataset , "ddxsdmedium"))
				prefix = "Med_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 100
			elseif(stringmatch(dataset , "ddxsdfast"))
				prefix = "Fast_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 100
			elseif(stringmatch(dataset , "ddxsternramp"))
				prefix = "Slow_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 30
			elseif(stringmatch(dataset , "hsv2slow"))
				prefix = "Slow_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 30
			elseif(stringmatch(dataset , "hsv2fast"))
				prefix = "Fast_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 30
			elseif(stringmatch(dataset , "inls69"))
				prefix = "Slow_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 30
			elseif(stringmatch(dataset , "inls68"))
				prefix = "Slow_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 30
			elseif(stringmatch(dataset , "inls42"))
				prefix = "Slow_Run_"
				extension = ".scl"
				filelowerbound = 0
				fileupperbound = 30
			endif






	string /g conditionpathname
	string conditionpath = conditionpathname

		NewPath/O/Q/M="Choose folder containing data files" conditionpath
		PathInfo/S conditionpath					// Check to see if user created the path.
		if (V_flag == 0)
			print "cancelled"
			return 0									// User cancelled.
		endif

	conditionpathname = conditionpath

	print S_path + " vflag: " + num2str(V_flag)

	string filepathname

	variable refnum

	string list
	string currentwave
	string ccwave
	variable ccwavelastpoint
	variable currentwavefirstpoint
	variable index
	variable deltapoints

	variable i
	for(i=filelowerbound;i<=fileupperbound;i+=1)	// initialize variables;continue test
		filepathname=""
		filepathname = S_path+prefix+num2str(i)+extension//need i to text
		refnum = 0
		open/R/Z=1 refnum filepathname
		close/A
		if(refnum != 0)
			if(stringmatch(dataset , "ddxbdslow"))
				DDXModelBDSLOWLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "ddxfldslow"))
				DDXModelFLDSLOWLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "ddxmedium"))
				DDXModelMEDIUMLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "ddxfast"))
				DDXModelFASTLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "ddxfld2slow"))
				DDXModelFLD2SLOWLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "ddxfld2medium"))
				DDXModelFLD2MEDIUMLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "ddxfld2fast"))
				DDXModelFLD2FASTLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "ddxsdmedium"))
				DDXModelSDMEDIUMLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "ddxsdfast"))
				DDXModelSDFASTLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "ddxsternramp"))
				DDXSternRampLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "hsv2slow"))
				HSV2slowLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "hsv2fast"))
				HSV2FASTLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "inls69"))
				INLSLoadFile(filepathname,0)
			elseif(stringmatch(dataset , "inls68"))
				INLSLoadFile2(filepathname,0)
			elseif(stringmatch(dataset , "inls42"))
				INLSLoadFile3(filepathname,0)
			endif
			if(i==filelowerbound)
				list = WaveList("*",";","")// A semicolon-separated list.
			endif

			index=0
			do
				// Get the next wave name
				currentwave = StringFromList(index, list)
				if (strlen(currentwave) == 0)
					break		// Ran out of waves
				endif
				//do something with the current wave
				if(singlechannel==-1 || index==singlechannel)
				ccwave = ""
				ccwave = "cc"+currentwave
				if(i == filelowerbound)
					if(stringmatch(splicemethod , "subtractmean"))
							SubtractMean(currentwave)
						endif
						duplicate/O $currentwave $(ccwave)
					else
						if(stringmatch(splicemethod , "subtractmean"))
							SubtractMean(currentwave)
						elseif(stringmatch(splicemethod , "forcecontinuous"))
							ccwavelastpoint = numpnts($ccwave)
							wave tempccwave = $ccwave
							wave tempwave = $currentwave
							ccwavelastpoint = 0
							ccwavelastpoint = (tempccwave[numpnts($ccwave)-1]+tempccwave[numpnts($ccwave)-2])/2
							currentwavefirstpoint = 0
							currentwavefirstpoint = (tempwave[0]+tempwave[1])/2
							deltapoints = currentwavefirstpoint - ccwavelastpoint
							//print "ccwlp: "+num2str(ccwavelastpoint)+" cwfp: "+num2str(currentwavefirstpoint)+" dp: "+num2str(deltapoints)
							tempwave = tempwave-deltapoints
						endif
						ConcatenateWaves(ccwave,currentwave)
					endif
				endif //end single channel if statement
				killwaves $currentwave
				index += 1
			while (1)
		Endif
	endfor											// execute body code until continue test is false

	if(stringmatch(dataset , "ddxbdslow"))
		DDXModelBDSLOWRenameScale("ccwave")
	elseif(stringmatch(dataset , "ddxfldslow"))
		DDXModelFLDSLOWRenameScale("ccwave")
	elseif(stringmatch(dataset , "ddxmedium"))
		DDXModelMEDIUMRenameScale("ccwave")
	elseif(stringmatch(dataset , "ddxfast"))
		DDXModelFASTRenameScale("ccwave")
	elseif(stringmatch(dataset , "ddxfld2slow"))
		DDXModelFLD2SLOWRenameScale("ccwave")
	elseif(stringmatch(dataset , "ddxfld2medium"))
		DDXModelFLD2MEDIUMRenameScale("ccwave")
	elseif(stringmatch(dataset , "ddxfld2fast"))
		DDXModelFLD2FASTRenameScale("ccwave")
	elseif(stringmatch(dataset , "ddxsdmedium"))
		DDXModelSDMEDIUMRenameScale("ccwave")
	elseif(stringmatch(dataset , "ddxsdfast"))
		DDXModelSDFASTRenameScale("ccwave")
	elseif(stringmatch(dataset , "ddxsternramp"))
		DDXSternRampRenameScale("ccwave")
	elseif(stringmatch(dataset , "hsv2slow"))
		HSV2slowRenameScale("ccwave")
	elseif(stringmatch(dataset , "hsv2fast"))
		HSV2FASTRenameScale("ccwave")
	elseif(stringmatch(dataset , "inls69"))
		INLSRenameScale("ccwave")
	elseif(stringmatch(dataset , "inls68"))
		INLSRenameScale2("ccwave")
	elseif(stringmatch(dataset , "inls42"))
		INLSRenameScale3("ccwave")
	endif


End//end nswccd65 general load from folder












function RemoveDropOutsInCurrentFolder()

end//remove drop outs


























//***********************************************************************************************************************
//	Proc GetWaveStatsInCurrentFolder(t1,t2)
//		This procedure gets the wave stats over a particular range of a wave.
Function SubtractMeanFromAllWaves()


	//Variable/G tbegin=t1,tend=t2
	String name = GetDataFolder(0)

	WaveStatListSM (WaveList("*",";",""),name)
End
//*****End of Procedure GetWaveStatsInCurrentFolder




//***********************************************************************************************************************
//	Function WaveStatList (list,name)
// 		This is a Hack since it works on all waves in current folder
// 		Use this command ===> WaveStatList (WaveList("*", ";", ""))
Function WaveStatListSM(list,name)
	String list, name
	String TheWave
	Variable tbegin, tend
	Variable index=0

	//Create a wave to hold the respective parameters
	Variable FinalCount

	Do
		// get the next wave name
		TheWave = GetStrFromList(list, index, ";")

		if (strlen(TheWave)==0)
			break
		else
          SubtractMean(TheWave)
		endif
		index += 1
	while (1)					// as long as condition is true


End
//*****End of Function WaveStatList





//***********************************************************************************************************************
//	Proc GetWaveStatsInCurrentFolder(t1,t2)
//		This procedure gets the wave stats over a particular range of a wave.
Function SubtractInitialFromAllWaves()


	//Variable/G tbegin=t1,tend=t2
	String name = GetDataFolder(0)

	WaveStatListSI (WaveList("*",";",""),name)
End
//*****End of Procedure GetWaveStatsInCurrentFolder




//***********************************************************************************************************************
//	Function WaveStatList (list,name)
// 		This is a Hack since it works on all waves in current folder
// 		Use this command ===> WaveStatList (WaveList("*", ";", ""))
Function WaveStatListSI(list,name)
	String list, name
	String TheWave
	Variable tbegin, tend
	Variable index=0

	//Create a wave to hold the respective parameters
	Variable FinalCount

	Do
		// get the next wave name
		TheWave = GetStrFromList(list, index, ";")

		if (strlen(TheWave)==0)
			break
		else
          SubtractInitial(TheWave)
		endif
		index += 1
	while (1)					// as long as condition is true


End
//*****End of Function WaveStatList




//this function was written for ddx to help
//subtract the mean from a run
//functions are faster than macros
//functions can be inserted into the macros menu though
Function SubtractInitial(wname)
	String wname //stores the selected wave name
	if(stringmatch( wname, ""))//check to see if we are calling this from the macro menu
		Prompt wname,"Wave",popup,WaveList("*", ";", "") //promps from waves in the working data folder
		variable V_Flag
		DoPrompt "Wave",wname //creates the prompt window
		if (V_Flag)
			print "cancelled"
			return 0	// check if user canceled
		endif
	endif

	variable scans = numpnts($wname) //get the number of scans from the selected wave

	wave waveref = $wname

	variable initialpt = waveref[0]

	waveref=waveref-initialpt

End //end Subtract Initial

















//***********************************************************************************************************************
//	Proc GetWaveStatsInCurrentFolder(t1,t2)
//		This procedure gets the wave stats over a particular range of a wave.
Function ResampleAllWaves()


	//Variable/G tbegin=t1,tend=t2
	String name = GetDataFolder(0)

	WaveStatListRS (WaveList("*",";",""),name)
End
//*****End of Procedure GetWaveStatsInCurrentFolder



//***********************************************************************************************************************
//	Function WaveStatList (list,name)
// 		This is a Hack since it works on all waves in current folder
// 		Use this command ===> WaveStatList (WaveList("*", ";", ""))
Function WaveStatListRS(list,name)
	String list, name
	String TheWave
	Variable tbegin, tend
	Variable index=0

	//Create a wave to hold the respective parameters
	Variable FinalCount

	Do
		// get the next wave name
		TheWave = GetStrFromList(list, index, ";")

		if (strlen(TheWave)==0)
			break
		else
          Resample(TheWave)
		endif
		index += 1
	while (1)					// as long as condition is true


End
//*****End of Function WaveStatList




//this function was written for ddx to help
//subtract the mean from a run
//functions are faster than macros
//functions can be inserted into the macros menu though
Function Resample(wname)
	String wname //stores the selected wave name
	if(stringmatch( wname, ""))//check to see if we are calling this from the macro menu
		Prompt wname,"Wave",popup,WaveList("*", ";", "") //promps from waves in the working data folder
		variable V_Flag
		DoPrompt "Wave",wname //creates the prompt window
		if (V_Flag)
			print "cancelled"
			return 0	// check if user canceled
		endif
	endif

	variable scans = numpnts($wname) //get the number of scans from the selected wave
	variable skippoints = 3

	wave waveref = $wname

	variable i
	for(i=0; i < numpnts(waveref)-skippoints;i+=1)
		deletepoints i,skippoints,waveref
	endfor

End //end Resample




















