//functions are faster than macros
//functions can be inserted into the macros menu
//Proc procedures can use global wave names directly
//proc are macros that do not show up in the menu by default



//this function uses the first derivative to find
//every oscillation in the time history
function selectrainflowamplitudes()

	variable /g samplerate
	variable tempsamplerate = samplerate
	prompt tempsamplerate,"Sample Rate"


	String /g wname //stores the selected wave name
	string tempwname = wname
	Prompt tempwname,"Wave",popup,WaveList("*", ";", "") //promps from waves in the working data folder
	variable V_Flag
	DoPrompt "Rainflow Amplitude Selection",tempwname,tempsamplerate //creates the prompt window
	if (V_Flag)
		print "cancelled"
		return 0	// check if user canceled
	endif

	wname = tempwname
	samplerate = tempsamplerate


	wave waveref = $wname


	variable scans = numpnts(waveref) //get the number of scans from the selected wave
	variable /g wmean
	wmean = mean(waveref,0,scans-1) //get the mean of the selected wave
	
	
	string amplitudeswave = "amplitudes"
	make/O $amplitudeswave
	deletepoints 0,numpnts($amplitudeswave),$amplitudeswave
	make /O/N=(scans) $amplitudeswave
	wave amplitudes = $amplitudeswave
	
	string amplitudeindexeswave = "amplitudeindexes"
	make/O $amplitudeindexeswave
	deletepoints 0,numpnts($amplitudeindexeswave),$amplitudeindexeswave
	make /O/N=(scans) $amplitudeindexeswave
	wave amplitudeindexes = $amplitudeindexeswave

	variable numberofamplitudes = 0
	
	variable i
	for(i=1;i<scans-1;i=i+1)
		if((waveref[i-1] >= waveref[i] && waveref[i+1] > waveref[i]) || (waveref[i-1] <= waveref[i] && waveref[i+1] < waveref[i]))
			amplitudes[numberofamplitudes] = waveref[i]
			amplitudeindexes[numberofamplitudes] = (i)/samplerate
			numberofamplitudes = numberofamplitudes + 1
		endif
	endfor

	redimension /N=(numberofamplitudes) amplitudes,amplitudeindexes
	print numberofamplitudes


	//display results
	display amplitudes vs amplitudeindexes as "Selected Amplitudes"
	appendtograph $wname
	ModifyGraph rgb($wname)=(0,12800,52224)
	ModifyGraph mode(amplitudes)=3
	Legend/C/N=text0/A=MC
	Label left "amplitude"
	Label bottom "time"
	TextBox/C/N=text1/A=MC "Amplitude Statistics"
	appendtext "numamplitudes: " + num2str(numpnts(amplitudes))

	performrainflow()
	applyMinersRule()

end //rainflow amplitudes





//this function uses rainflow algorithm 1 from
//Simple rainflow counting algorithms   ARTICLE
//International Journal of Fatigue, Volume 4, Issue 1, January 1982, Pages 31-40
//S. D. Downing and D. F. Socie
//this is also algorithm 5.4.5 Simplified Rainflow from ASTM E 1049-85
function performrainflow()


	//rearrange amplitudes to begin and end with maximum peak
	//variable/g V_maxloc
	//variable/g V_npnts
	wave amplitudes
	wavestats amplitudes
	
	make/O rainflowamplitudes
	deletepoints 0,numpnts(rainflowamplitudes),rainflowamplitudes
	Duplicate/R=(V_maxloc,)/O amplitudes rainflowamplitudes
	
	variable i
	for(i=0;i<=V_maxloc;i=i+1)
		InsertPoints numpnts(rainflowamplitudes),1, rainflowamplitudes
		rainflowamplitudes[numpnts(rainflowamplitudes)-1] = amplitudes[i]
	endfor
	
	SetScale/P x 0,1,"", rainflowamplitudes
	
	//start rainflow algorithm 1
	variable x, y, step=1
	//variable avgy,avgx
	make/O tempcycles
	deletepoints 0,numpnts(tempcycles),tempcycles
	make/O rawcycles
	deletepoints 0,numpnts(rawcycles),rawcycles
	i=0
	do
		//step 1
		if(step == 1)
			if(i < numpnts(rainflowamplitudes))
				InsertPoints numpnts(tempcycles),1, tempcycles
				tempcycles[numpnts(tempcycles)-1] = rainflowamplitudes[i]
				i=i+1
				step = 2
			else
				step = 5
			endif
		endif
		
		//step 2
		if(step == 2)
			if(numpnts(tempcycles) < 3)
				step = 1
			else
				y = abs(tempcycles[numpnts(tempcycles)-3] - tempcycles[numpnts(tempcycles)-2])
				x = abs(tempcycles[numpnts(tempcycles)-2] - tempcycles[numpnts(tempcycles)-1])
				if(x == 0 || y == 0)
					DeletePoints numpnts(tempcycles)-2,1, tempcycles
					step = 2
				else
					step = 3
				endif
			endif
		endif
		
		//step 3
		if(step == 3)
			if( x < y)
				step = 1
			else
				step = 4
			endif
		endif
		
		
		//step 4
		if(step == 4)
			//print "cycle: " + num2str(y)
			InsertPoints numpnts(rawcycles),1, rawcycles
			rawcycles[numpnts(rawcycles)-1] = y
			DeletePoints numpnts(tempcycles)-3,2, tempcycles
			step = 2
		endif
	
	while(step != 5)
	
	
	print "Found " + num2str(numpnts(rawcycles)) + " cycles."
	
	//display results
	display rawcycles as "Raw Cycles"
	

end //perform rainflow













//this function uses 5.4.4 Rainflow Counting from ASTM E 1049-85
function performrainflowAlgorithm2()

	wave amplitudes
	wavestats amplitudes
	
	make/O rainflowamplitudes
	deletepoints 0,numpnts(rainflowamplitudes),rainflowamplitudes
	Duplicate/O amplitudes rainflowamplitudes
	
	variable i
	
	SetScale/P x 0,1,"", rainflowamplitudes
	
	//start rainflow algorithm 5.4.4
	variable x, y, step=1
	make/O tempcycles
	deletepoints 0,numpnts(tempcycles),tempcycles
	make/O rawcycles
	deletepoints 0,numpnts(rawcycles),rawcycles
	make/O halfcycles
	deletepoints 0,numpnts(halfcycles),halfcycles
	i=0
	do
		//step 1
		if(step == 1)
			if(i < numpnts(rainflowamplitudes))
				InsertPoints numpnts(tempcycles),1, tempcycles
				tempcycles[numpnts(tempcycles)-1] = rainflowamplitudes[i];
				i=i+1
				step = 2
			else
				step = 6
			endif
			//print "ra: " + num2str(rainflowamplitudes[i])
		endif
		
		
		//step 2
		if(step == 2)
			if(numpnts(tempcycles) < 3)
				step = 1
			else
				//print num2str(numpnts(rainflowamplitudes))
				//y = abs(tempcycles[numpnts(tempcycles)-2] - tempcycles[numpnts(tempcycles)-3]);
				y = abs(tempcycles[numpnts(tempcycles)-3] - tempcycles[numpnts(tempcycles)-2]);
				//print num2str(rainflowamplitudes[i+1])
				//print num2str(rainflowamplitudes[i])
				//x = abs(tempcycles[numpnts(tempcycles)-1] - tempcycles[numpnts(tempcycles)-2]);
				x = abs(tempcycles[numpnts(tempcycles)-2] - tempcycles[numpnts(tempcycles)-1]);
				//print num2str(rainflowamplitudes[i+2])
				//print num2str(rainflowamplitudes[i+1])
				if(x == 0)
					DeletePoints numpnts(tempcycles)-2,1, tempcycles
					step = 2
				else
					step = 3
				endif
			endif
		endif
		
		
		//step 3
		if(step == 3)
			if( x < y)
				step = 1
			else
				step = 4
			endif
		endif
		
		
		//step 4
		if(step == 4)
			if(numpnts(tempcycles) == 3)
				step = 5
			else
				InsertPoints numpnts(rawcycles),1, rawcycles
				rawcycles[numpnts(rawcycles)-1] = y
				DeletePoints numpnts(tempcycles)-3,2, tempcycles
				step = 2
			endif
		endif
		
		
		//step 5
		if(step == 5)
			InsertPoints numpnts(halfcycles),1, halfcycles
			halfcycles[numpnts(halfcycles)-1] = y
			DeletePoints numpnts(tempcycles)-3,1, tempcycles
			step = 2
		endif
		
		
		//step 6
		if(step == 6)
			print numpnts(tempcycles)
			if(numpnts(tempcycles) > 1)
				do
					InsertPoints numpnts(halfcycles),1, halfcycles
					x = abs(tempcycles[numpnts(tempcycles)-2] - tempcycles[numpnts(tempcycles)-1]);
					halfcycles[numpnts(halfcycles)-1] = x
					DeletePoints numpnts(tempcycles)-1,1, tempcycles
				while(numpnts(tempcycles) > 1)
			endif
		endif
	
	while(step != 6)
	
	
	print "Found " + num2str(numpnts(rawcycles)) + " cycles."
	
	//display results
	display rawcycles as "Raw Cycles"
	

end //perform rainflow

















//accumulates damage of varying stresses/strains
//http://en.wikipedia.org/wiki/Metal_fatigue
function applyMinersRule()

	//define the SN curve for the desired material
	//brittle aluminum with uts of 320MPa
	//320,1
	//300,10
	//260,100
	//200,1000
	//150,10000
	//120,100000
	//95,1000000
	//80,10000000
	//2 waves one called stress, one called cycles

	make/O stress
	deletepoints 0,numpnts(stress),stress
	make/O cycles
	deletepoints 0,numpnts(cycles),cycles
	make/O/N=9 stress cycles
	stress[0] = 320
	stress[1] = 300
	stress[2] = 260
	stress[3] = 200
	stress[4] = 150
	stress[5] = 120
	stress[6] = 95
	stress[7] = 80
	stress[8] = 50
	cycles[0] = 1e0
	cycles[1] = 1e1
	cycles[2] = 1e2
	cycles[3] = 1e3
	cycles[4] = 1e4
	cycles[5] = 1e5
	cycles[6] = 1e6
	cycles[7] = 1e7
	cycles[8] = 1e8


	Display stress vs cycles
	ModifyGraph log(bottom)=1
	ModifyGraph mode=4

	//turn the rawcycles into a histogram; possibly compute the average of each bin
	//binning might not be necessary

	wave rawcycles
	//just for testing to get test data into sn curve range
	//rawcycles = rawcycles/500


	//apply Miner's Rule and correlate the damage at each stress level
	variable i, j, C=0, amplitude, ub, ubx, lb, lbx, m, b, N
	for(i=0; i < numpnts(rawcycles); i=i+1)
		amplitude = rawcycles[i]
		if(amplitude >= stress[numpnts(stress)])
			//find number of cycles asociated with amplitude
			for(j=0; j < numpnts(stress)-1;j=j+1)
				if(amplitude <= stress(j) && amplitude > stress(j+1))
					ub = stress(j)
					ubx = cycles(j)
					lb = stress(j+1)
					lbx = cycles(j+1)
					break
				endif
			endfor
			m = (ub-lb)/(ubx-lbx)
			b = ub-m*ubx
			N = (amplitude-b)/m
			C = C + 1/N
		endif
	endfor
	
	print "Sum Damage, C: " + num2str(C)


end //applyMinersRule















